import requests


class TorpedoClient:
    """Cliente HTTP para comunicação com Torpedo

    :param str torpedo_url: Endereço da API do serviço Torpedo
    :param str client_name: Nome do cliente requisitando o envio do email,
        esse parâmetro será usado em conjunto com o ``template_name`` para encontrar o template do email
    """

    def __init__(self, torpedo_url, client_name):

        self.torpedo_url = torpedo_url
        self.client_name = client_name

    def send_email(self, email, subject, template_name, data=None):
        """Envia email com um template pré-definido do Torpedo através da API do `Mailgun <https://www.mailgun.com/>`_.

        Exemplo:

        .. code-block:: python

            from stl_sdk.torpedo import TorpedoClient

            client = TorpedoClient('https://notifications.spacetimeanalytics.com', 'waves')
            response = client.send_email('pedro@gmail.com', 'Bem-vindo!', 'welcome', { 'name': 'Pedro' })

        :param email: Email do destinatário
        :type email: str
        :param subject: Assunto do email
        :type subject: str
        :param template_name: Nome do template do Torpedo
            (`Veja os templates disponíveis <http://notifications.spacetimeanalytics.com>`_).
        :type template_name: str
        :param data: Dicionário de dados para popular o email
        :type data: dict, optional
        :return: Resposta da requisição para o Mailgun
        :rtype: requests.Response
        """
        response = requests.post(
                '{}/api/send-email'.format(self.torpedo_url),
                json={
                    'to_email': email,
                    'subject': subject,
                    'template_name': template_name,
                    'data': data,
                    'client': self.client_name,
                }
            )
        response.raise_for_status()
        return response



import jwt
import requests


class AtlantisClient:
    """Cliente HTTP para comunicação com Atlantis

    :param str atlantis_url: Endereço da API do serviço Atlantis
    :param str client_id: ID do cliente cadastrado no Atlantis
    :param str client_secret: Secret do cliente cadastrado no Atlantis
    """

    def __init__(self, client_id=None, client_secret=None, atlantis_url='https://accounts.spacetimeanalytics.com'):
        self.atlantis_url = atlantis_url
        self._client_id = client_id
        self._client_secret = client_secret

    @property
    def client_id(self):
        if self._client_id is None:
            raise InvalidInitParameters('Missing ATLANTIS_CLIENT_ID on AtlantisClient initialization.')
        return self._client_id

    @property
    def client_secret(self):
        return self._client_secret

    def issue_token(self, code, redirect_uri):
        """Realiza a troca do code pelo token

        .. deprecated:: 0.0.8
            Use a biblioteca authlib.

        Exemplo:

        .. code-block:: python

            from stl_sdk.atlantis import AtlantisClient

            client = AtlantisClient('https://accounts.spacetimeanalytics.com', 'client_id_abc123', 'client_secret_123abc')
            token = client.issue_token('code123abc')

        :param code: Código concedido pelo Atlantis via redirect (authorization_code)
        :type code: str
        :param redirect_uri: Endereço de redirecionamento
        :type redirect_uri: str
        :return: Dicionário com os tokens: ``access_token``, ``id_token`` e ``refresh_token``
        :rtype: dict
        """
        response = requests.post(
            '{}/api/token'.format(self.atlantis_url),
            data={
                'code': code,
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'redirect_uri': redirect_uri,
                'grant_type': 'authorization_code'
            })
        response.raise_for_status()
        return response.json()

    def get_public_key(self):
        """Obtém a chave pública do Atlantis para validação do token de acesso

        .. deprecated:: 0.0.8
            Use a biblioteca authlib.

        :return: Chave pública
        :rtype: str
        """
        response = requests.get('{}/api/jwks'.format(self.atlantis_url))
        response.raise_for_status()
        return response.json()

    def validate_token(self, id_token):
        """Valida token de acesso do usuário

        .. deprecated:: 0.0.8
            Use a biblioteca authlib.

        Exemplo:

        .. code-block:: python

            from stl_sdk.atlantis import AtlantisClient

            client = AtlantisClient('https://accounts.spacetimeanalytics.com', 'client_id_abc123', 'client_secret_123abc')
            token_payload = client.validate_token('tokenJwT')

        :param id_token: Token JWT do usuário gerado pelo Atlantis
        :type id_token: str
        :raises InvalidTokenError: Se o token estiver expirado ou alterado
        :return: Retorna o payload do token JWT
        :rtype: dict
        """
        public_key = self.get_public_key()

        try:
            return jwt.decode(id_token, public_key, audience=self.client_id)
        except Exception as err:
            raise InvalidTokenError(err)

    def introspect_token(self, access_token, raise_for_inactive=True):
        """Verifica se o access_token está válido

        :param access_token: Token do usuário gerado pelo Atlantis
        :type access_token: str
        :param raise_for_inactive: Retorna um erro caso o token esteja inativo, valor padrão `True`
        :type access_token: bool, opcional
        :return: Retorna dados do token: ``active``, ``email``, ``scope``, ``client_id``, ``exp``, etc..
        :rtype: dict
        """
        response = requests.post(
            '{}/api/introspect-token'.format(self.atlantis_url),
            data={
                'token': access_token,
                'token_type_hint': 'access_token',
                'client_id': self.client_id,
                'client_secret': self.client_secret
            })
        response.raise_for_status()

        json = response.json()

        if raise_for_inactive and json['active'] is False:
            raise InvalidTokenError('Inactive token')

        return json

    def introspect_api_key(self, api_key):
        """Verifica se a Api key é válida

        :param api_key: Api key gerada pelo Atlantis
        :type api_key: str
        :return: Retorna dados da api key: ``actions``, ``description``, ``created_at``, etc..
        :rtype: dict
        """
        response = requests.post(
            '{}/api/introspect-apikey'.format(self.atlantis_url),
            data={
                'apikey': api_key,
                'client_id': self.client_id,
                'client_secret': self.client_secret
            })
        response.raise_for_status()
        return response.json()


class AtlantisClientFlask(AtlantisClient):

    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)
        super().__init__()

    def init_app(self, app):
        self.atlantis_url = app.config['ATLANTIS_URL']
        self._client_id = app.config['ATLANTIS_CLIENT_ID']
        self._client_secret = app.config.get('ATLANTIS_CLIENT_SECRET')


class InvalidTokenError(RuntimeError):
    pass


class InvalidInitParameters(RuntimeError):
    pass



import requests
from . import VERSION


class WavesClient:
    """Cliente HTTP para comunicação com Waves

    :param str waves_url: Endereço da API do Waves
    :param str api_key: Chave de API para autenticação
    """

    def __init__(self, waves_url, api_key):
        self.waves_url = waves_url
        self.api_key = api_key

    def get_workflow(self, identifier):
        """Obtem um fluxo por ID

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            workflow = client.get_workflow('aweeds@v1.0.2')

        :param identifier: Id do fluxo desejado
        :type identifier: str
        :return: Objeto do fluxo
        :rtype: dict
        """
        response = requests.get(
            '{}/api/workflows/{}'.format(self.waves_url, identifier),
            headers=self._get_headers(),
        )
        response.raise_for_status()
        return response.json()

    def create_workflow(self, workflow):
        """Cria um novo fluxo

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            workflow = client.create_workflow({
                'name': 'workflow',
                'steps': []
            })

        :param workflow: Fluxo a ser criado
        :type workflow: dict
        :return: Objeto do fluxo criado
        :rtype: dict
        """
        response = requests.post(
            '{}/api/workflows'.format(self.waves_url),
            headers=self._get_headers(),
            json=workflow)
        response.raise_for_status()
        return response.json()

    def archive_workflow(self, identifier):
        """Arquiva um fluxo

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            workflow = client.archive_workflow('workflow_identifier')

        :param identifier: Id do fluxo desejado
        :type identifier: str
        :return: Objeto do fluxo arquivado
        :rtype: dict
        """
        response = requests.put(
            '{}/api/workflows/{}/archive'.format(self.waves_url, identifier),
            headers=self._get_headers())
        response.raise_for_status()
        return response.json()

    def unarchive_workflow(self, identifier):
        """Desarquiva um fluxo

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            workflow = client.unarchive_workflow('workflow_identifier')

        :param identifier: Id do fluxo desejado
        :type identifier: str
        :return: Objeto do fluxo desarquivado
        :rtype: dict
        """
        response = requests.put(
            '{}/api/workflows/{}/unarchive'.format(self.waves_url, identifier),
            headers=self._get_headers())
        response.raise_for_status()
        return response.json()

    def list_executions(self, term=None, status=None, sort_by=None, page=None, size=None):
        """Busca execuções com os filtros passados

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            executions = client.list_executions()

        :param term: Palavra-chave de busca (Identificador, descrição ou fluxo)
        :type term: str, opcional
        :param status: Lista de status desejados (Status possíveis: QUEUED, SUCCESS, CANCELED, FAILURE, STARTED) 
        :type status: list, opcional
        :param sort_by: Nome da coluna para ordenação. Utilize "-" para ordem decrescente
        :type sort_by: str, opcional
        :param page: Número da página atual 
        :type page: int, opcional
        :param size: Tamanho da página
        :type size: int, opcional
        :return: Objeto da paginação contendo as chaves "meta" e "objects"
        :rtype: dict
        """
        response = requests.get(
            '{}/api/executions'.format(self.waves_url),
            headers=self._get_headers(),
            params={
                'term': term, 
                'status': status,
                'sort_by': sort_by, 
                'page': page, 
                'size': size
            }
        )
        response.raise_for_status()
        return response.json()

    def get_execution(self, identifier):
        """Obtem uma execução por ID

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            execution = client.get_execution('Jun-26-8857')

        :param identifier: Id da execução desejada
        :type identifier: str
        :return: Objeto da execução
        :rtype: dict
        """
        response = requests.get(
            '{}/api/executions/{}'.format(self.waves_url, identifier),
            headers=self._get_headers(),
        )
        response.raise_for_status()
        return response.json()

    def create_execution(self, workflow_identifier, inputs, description=None):
        """Cria uma execução a partir de um fluxo e uma lista de entradas

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            execution = client.create_execution('aweeds@v1.0.2', {'shapefile': 's3://file.zip'})

        :param workflow_identifier: Id do fluxo
        :type workflow_identifier: str
        :param inputs: Entradas do fluxo no formato chave:valor
        :type inputs: dict
        :param description: Descrição da execução
        :type description: str, opcional
        :return: Objeto da execução criada
        :rtype: dict
        """
        response = requests.post(
            '{}/api/executions'.format(self.waves_url),
            headers=self._get_headers(),
            json={
                "workflow_identifier": workflow_identifier,
                "inputs": inputs,
                "description": description
            })
        response.raise_for_status()
        return response.json()

    def update_execution(self, identifier, description=None):
        """Atualiza os dados de uma execução. Atualmente só é possível atualizar a sua descrição

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            execution = client.update_execution('Jun-26-8331', 'nova descrição')

        :param identifier: Id da execução
        :type identifier: str
        :param description: Nova descrição da execução
        :type description: str
        :return: Objeto da execução atualizada
        :rtype: dict
        """
        response = requests.put(
            '{}/api/executions/{}'.format(self.waves_url, identifier),
            headers=self._get_headers(),
            json={
                "description": description
            })
        response.raise_for_status()
        return response.json()

    def delete_execution(self, identifier):
        """Realiza um SOFT DELETE na execução desejada

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            client.delete_execution('Jun-26-8331')

        :param identifier: Id da execução
        :type identifier: str
        """
        response = requests.delete(
            '{}/api/executions/{}'.format(self.waves_url, identifier),
            headers=self._get_headers())
        response.raise_for_status()

    def create_businesstask(self, btask):
        """Cria uma nova tarefa

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            btask = client.create_businesstask({
                'name': 'btask',
                'runs_on': 'awsasg',
                'docker_registry': 'ubuntu:latest'})

        :param btask: Tarefa a ser criada
        :type btask: dict
        :return: Objeto da tarefa criada
        :rtype: dict
        """
        response = requests.post(
            '{}/api/business-tasks'.format(self.waves_url),
            headers=self._get_headers(),
            json=btask)
        response.raise_for_status()
        return response.json()

    def publish_businesstask(self, identifier):
        """Pública uma tarefa não publicada

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            btask = client.publish_businesstask('btask_identifier')

        :param identifier: Id da tarefa
        :type identifier: str
        :return: Objeto da tarefa publicada
        :rtype: dict
        """
        response = requests.put(
            '{}/api/business-tasks/{}/publish'.format(self.waves_url, identifier),
            headers=self._get_headers())
        response.raise_for_status()
        return response.json()

    def unpublish_businesstask(self, identifier):
        """Despública uma tarefa publicada

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            btask = client.unpublish_businesstask('btask_identifier')

        :param identifier: Id da tarefa
        :type identifier: str
        :return: Objeto da tarefa despublicada
        :rtype: dict
        """
        response = requests.put(
            '{}/api/business-tasks/{}/unpublish'.format(self.waves_url, identifier),
            headers=self._get_headers())
        response.raise_for_status()
        return response.json()

    def list_workers(self):
        """Lista os workers da aplicação

        Exemplo:

        .. code-block:: python

            from stl_sdk.waves import WavesClient

            client = WavesClient('https://waves-dev.spacetimeanalytics.com', 'api_key')
            workers = client.list_workers()

        :return: Objeto dos workers da aplicação
        :rtype: dict
        """
        response = requests.get(
            '{}/api/workers'.format(self.waves_url),
            headers=self._get_headers(),
        )
        response.raise_for_status()
        return response.json()

    def _get_headers(self):
        headers = {
            'User-Agent': 'stl-sdk/{}'.format(VERSION),
            'X-API-KEY': self.api_key
        }
        return headers



import requests


class CoralClient:
    """Cliente HTTP para comunicação com Coral

    :param str coral_url: Endereço da API do serviço Coral
    :param str api_key: Chave de acesso a API
    :param str atlantis_client_id: Id do cliente que a chave de API foi emitida
    """

    def __init__(self, coral_url, api_key, atlantis_client_id):
        self.coral_url = coral_url
        self.api_key = api_key
        self.atlantis_client_id = atlantis_client_id

    def _get_headers(self, bucket, region=None):
        return {
            'bucket': bucket,
            'region': region,
            'atlantisclientid': self.atlantis_client_id,
            'x-api-key': self.api_key
        }

    def get_file(self, bucket, region, key):
        """Obtém informações de um determinado arquivo.

        Exemplo:

        .. code-block:: python

            from stl_sdk.torpedo import TorpedoClient

            client = CoralSyncClient('https://storage.spacetimeanalytics.com', 'apikey', 'atlantisclientid')
            file = client.get_file('spacetimelabs-bucket', 'us-west-2', '/folder/file.ext')

        :param bucket: Bucket do arquivo
        :type bucket: str
        :param region: Região do bucket
        :type region: str
        :param key: Chave do arquivo
        :type key: str
        :return: Objeto do arquivo
        :rtype: dict
        """
        response = requests.get(
                '{}/api/file'.format(self.coral_url),
                headers=self._get_headers(bucket, region=region),
                params={
                    'key': key
                }
            )
        response.raise_for_status()
        return response.json()


class CoralSyncClient(CoralClient):
    """Cliente HTTP para sincronização dos arquivos do banco de dados do Coral

    :param str coral_url: Endereço da API do serviço Coral
    :param str api_key: Chave de acesso a API
    :param str atlantis_client_id: Id do cliente que a chave de API foi emitida
    """

    def __init__(self, coral_url, api_key, atlantis_client_id):
        super().__init__(coral_url, api_key, atlantis_client_id)

    def upsert_file(self, bucket, key, last_modified, size, metadata=None):
        """Crie ou atualiza um arquivo no banco do Coral.

        Exemplo:

        .. code-block:: python

            from stl_sdk.torpedo import TorpedoClient

            client = CoralSyncClient('https://storage.spacetimeanalytics.com', 'apikey', 'atlantisclientid')
            file_uri = client.upsert_file('spacetimelabs-bucket', '/folder/file.ext', '2020-06-26T14:51Z', 123,
                                          { 'thumb': 's3://bucket/folder/file.ext' })

        :param bucket: Bucket do arquivo
        :type bucket: str
        :param key: Chave do arquivo
        :type key: str
        :param last_modified: Data de última modificação do arquivo (Formato datetime ISO)
        :type last_modified: str
        :param size: Tamanho do arquivo em bytes
        :type size: int
        :param metadata: Dicionário de metadados do arquivo
        :type metadata: dict, optional
        :return: Objeto do arquivo persistido
        :rtype: dict
        """
        response = requests.post(
                '{}/api/sync'.format(self.coral_url),
                headers=self._get_headers(bucket),
                params={
                    'key': key
                }, json={
                    'last_modified': last_modified,
                    'size': size,
                    'metadata': metadata,
                }
            )
        response.raise_for_status()
        return response.json()

    def delete_file(self, bucket, key):
        """Remove um arquivo do banco do Coral.

        Exemplo:

        .. code-block:: python

            from stl_sdk.torpedo import TorpedoClient

            client = CoralSyncClient('https://storage.spacetimeanalytics.com', 'apikey', 'atlantisclientid')
            client.delete_file('spacetimelabs-bucket', '/folder/file.ext')

        :param bucket: Bucket do arquivo
        :type bucket: str
        :param key: Chave do arquivo
        :type key: str
        :return: Código da resposta (204 para successo)
        :rtype: int
        """
        response = requests.delete(
                '{}/api/sync'.format(self.coral_url),
                headers=self._get_headers(bucket),
                params={
                    'key': key
                }
            )
        response.raise_for_status()
        return response.status_code
